//Direction.cpp

#include "Direction.h"

Direction::Direction()
{
	signal = RED;
	total_throughput = 0;
	current_time = 0;
	vpm = 0;
	current_throughput = 0;
	time_last_change = 0;
	total_avg_wait = 0;
}

Direction::~Direction()
{
	while (!carQ.empty())
		carQ.pop_front();
}

//sets the signal to the given int
void Direction::setSignal(int s)
{ 
	time_last_change = current_time;
	signal = s;
}   

//return true if signal is red, false otherwise
bool Direction::signalRed()
{
	return (signal == RED);
}

//return true if signal is gree, false otherwise
bool Direction::signalGreen()
{
	return (signal == GREEN);
} 
	
//create a new car with given arrival_time
void Direction::addCar(u_int t)
{
	carQ.push_back(Car(t));
}   


//sets this direction's cars per minute rate
void Direction::setVPM(double v)
{
	vpm = v;
}         


//return the total throughput of this direction
u_int Direction::getTotalThroughput()
{
	return total_throughput;
}

//return the total traffic density
double Direction::getTrafficDensity()
{
	if (time_last_change != current_time)
		return ( (double) current_throughput / (double) ((double) (current_time - time_last_change) / (double) 60));
	else
		return (0);
}



//return the averge waiting time for cars waiting
double Direction::getAverageWait()
{
	double total_wait = 0;
	deque<Car>::size_type i = 0;

	for (deque<Car>::iterator it = carQ.begin(); it < carQ.end(); ++it)
	{
		total_wait += (current_time - (*it).getArrivalTime()) + i;
		++i;
	}
	
	if (total_wait == 0)
		return 0;
	else
		return (total_wait / (double) carQ.size());

	
	
/*	double total_wait = 0;

	for (deque<Car>::iterator it = carQ.begin(); it < carQ.end(); ++it)
	{
		total_wait += (current_time - (*it).getArrivalTime());
	}
	
	if (total_wait == 0)
		return 0;
	else
		return (total_wait / (double) carQ.size());
*/
}

//return the total average wait
double Direction::getTotalAverageWait()
{
	return total_avg_wait;
}


//execute the following during each cycle
void Direction::doStuff()
{
	//increment the current time value
	++current_time;
	
	//determine if a car is arriving at the light
	if (testForCarArrival())
	{
		addCar(current_time);
	}

	//test if the signal is red
	if (signalRed())
	{	//the signal is red, do red light stuff
		//set the current throughput to 0
		current_throughput = 0;
	}
	else if (signalGreen())
	{	//the signal is green
		//test if there are any cars
		if (!carQ.empty())
		{
			//calculate the average wait
			double this_wait = (current_time - carQ.front().getArrivalTime());
			total_avg_wait =   ( ((total_avg_wait * (double) total_throughput) + this_wait) / ((double) (total_throughput + 1)) );
			
			//calculate the car coming through
			total_throughput ++;
			current_throughput ++;
				
			//remove one of the waiting cars
			carQ.pop_front();
		}
	}
}


//returns true if a car should arrive
bool Direction::testForCarArrival()
{
	return ( (rand() % 60) < vpm);
}


void Direction::reset()
{
	signal = RED;
	total_throughput = 0;
	current_time = 0;
	vpm = 0;
	current_throughput = 0;
	time_last_change = 0;
	total_avg_wait = 0;
	while (!carQ.empty())
		carQ.pop_front();
}



#if #debug(yes)
void Direction::show()
{
	std::cout << " sig ("    << signal             << ")";
	std::cout << ", vpm ("   << vpm                << ")";
	std::cout << ", ctime (" << current_time       << ")";
	std::cout << ", tlc ("   << time_last_change   << ")";
	std::cout << ", ctp ("   << current_throughput << ")";
	std::cout << ", ttp ("   << total_throughput   << ")";
	std::cout << std::endl;
	for (deque<Car>::size_type i = 0; i < carQ.size(); ++i)
	{
		std::cout <<"Car " << i << ": ";
		carQ[i].show();
		std::cout << std::endl;
	}
}
#endif
