//myStringFunctions.c

/*
Steve Bono
(4655)
11-2-2002

Lab05
*/

#include "myStringFunctionsCPP.h"

//--------------------------------------------------------------------------
// Converters
//--------------------------------------------------------------------------

/* NAME: intToString
 * TYPE: char *
 * ARGS: int n - integer to be converted to a string
 * RETURNS: char *s - a string version of the integer supplied
 * DESCRIPTION: uses the sprintf function to convert an integer into a string.
 */
std::string& intToString(int n, std::string &s)
{
	char temp[32];
	sprintf(temp,"%d",n);
	s = temp;
	return (s);
}


/* NAME: stringToInt
 * TYPE: int
 * ARGS: std::string s - string to convert
 * RETURNS: integer value represented by the string
 * DESCRIPTION: converts a string into an integer value.
 * Returns -9999 on error
 */
int stringToInt(std::string s)
{
	return( (int) stringToLong(s) );
}


/* NAME: stringToLong
 * TYPE: int
 * ARGS: string s - string being converted
 * RETURNS: long value represented by the string
 * DESCRIPTION: converts a string into an long value.
 * Returns -9999 on error
 */
long stringToLong(std::string s)
{
	if ( ((s[0] > '9') || (s[0] < '0')) && (s[0] != '-'))
		return(-9999);
	for (std::string::iterator it = s.begin()+1; it < s.end(); ++it)
		if (((*it) > '9') || ((*it) < '0'))
			return(-9999);

	int retval = 0;
	for (std::string::iterator it = s.begin(); it < s.end(); ++it)
		if ((*it) != '-')
			retval = ( (retval * 10) + ((*it) - '0') );
	
	if (s[0] == '-')
		retval *= -1;
		
	return(retval);
}


/* NAME: stringToShort
 * TYPE: short
 * ARGS: string s - string to be converted
 * RETURNS: short integer value represented by the string
 * DESCRIPTION: converts a string into a short integer value.
 * Returns -9999 on error
 */
short stringToShort(std::string s)
{
	return((short) stringToLong(s));
}

//NAME: intToChar
//DESCRIPTION: returns a character value for an integer
char intToChar(int i)
{
	if ( (i < 0) || (i > 9) )
		return(-1);
	return ( (char) (i + '0'));
}

//NAME: charToInt
//DESCRIPTION: returns an integer value of a character
int charToInt(char c)
{
	if ( (c < '0') || (c > '9') )	//return a -1 if c is not an integer
		return(-1);			//retpresentation
	return ( (int) (c - '0') );
}


//-----------------------------------------------------------------------
// Prompts
//-----------------------------------------------------------------------

//NAME: intPrompt()
//DESCRIPTION: prompts the user for an integer value
int intPrompt()
{
	std::string s;
	std::cin >> s;
	return( stringToInt(s));
}


//NAME: longPrompt()
//DESCRIPTION: prompts user for a long value
long longPrompt()
{
	std::string s;
	std::cin >> s;
	return( stringToLong(s) );

}


//NAME: shortPrompt()
//DESCRIPTION: prompts the user for a short int value
short shortPrompt()
{
	std::string s;
	std::cin >> s;
	return(stringToShort(s));
}


//NAME: charPrompt()
//DESCRIPTION: prompts the user to enter a character
char charPrompt()
{
	std::string s;
	std::cin >> s;
	return(s[0]);
}


//NAME: stringPrompt()
//DESCRIPTION: prompts the user to enter a string
std::string &stringPrompt(std::string &s)
{
	std::cin >> s;
	return(s);
}

//-----------------------------------------------------------------------
// Other
//-----------------------------------------------------------------------

//NAME: printTabs
//DESCRIPTION: prints the inputed number of tabs
void printTabs(int n)
{ 	int i;	for (i=0;i<n;i++) printf("\t");	}



//NAME: printEnters
//DESCRIPTION: prints the inputed number of enters
void printEnters(int n)
{	int i;	for (i=0;i<n;i++) printf("\n");	}



//NAME: anykey
//DESCRIPTION: awaits the keypress of any key
void anykey()
{
	fflush(stdin);
	char s[32];
	fgets(s,32,stdin);
}

//NAME: getTimestamp
//DESCRIPTION: generates a timestamp and returns it as a string
// char *ts must be at a minimum of TIMESTAMP_SIZE characters long.
std::string& getTimestamp(std::string &ts)
{
	char temp[32];
	_strtime(temp);
	ts = temp;
	return ts;
}
