//Rule.h

#ifndef RULE_H
#define RULE_H

#include "FuzzySet.h"

#if #debug(yes)
	#include <iostream>
	#include <iomanip>
#endif

template<class T1, class T2, class T3> class RuleBase;

template<class T1, class T2, class T3>
class Rule
{
public:
	//default constructor (needed for map)
	Rule() : A_set((T1) 0), B_set((T2) 0), A_membership(0), B_membership(0) {}
	//constructor with arguments
	Rule(T1 a1, T2 b1, double a2, double b2) :
		A_set(a1), B_set(b1), A_membership(a2), B_membership(b2) {}
	//copy constructor
	Rule(const Rule &rhs) :
		A_set(rhs.A_set), B_set(rhs.B_set), A_membership(rhs.A_membership), B_membership(rhs.B_membership) {}
	//destructor
	~Rule() {}
	
	//assignment operator (needed for map)
	Rule &operator=(Rule &rhs);
	//less than comparision operator (needed for map)
	bool operator<(const Rule &rhs) const;
	
	//returns the output value of this rule
	T3 getOutput();
	//returns the fire strength of this rule
	double getFireStrength();
	
	friend class RuleBase<T1,T2,T3>;
	
#if #debug(yes)
	void show();
#endif

protected:	
	T1 A_set;
	T2 B_set;
	double A_membership;
	double B_membership;
};

template class Rule<FuzzyInput_A, FuzzyInput_B, FuzzyOutput>;


#endif
