//IntersectionSimulator.cpp

#include "IntersectionSimulator.h"

int main()
{
	                      //declare local variables
	u_int current_time;   //the current simulation time
	u_int finish_time;    //the finish time for this simulation
	u_int timer_interval; //used if the controller type is a timer
	
	u_int seed;          //the seed for the random number generator
	int controller_type; //the controller type to use
	
	Direction d1;        //directions used in the simulation
	Direction d2;
	
	FuzzyLogicController flc; //the fuzzy logic controller used in this simulation
	
	double vpm1, vpm2;   //the vehicles per minute (VPM)
	int vpm_type;        //the VPM change type to use
	
	double vpm_low1;
	double vpm_low2;
	double vpm_high1;
	double vpm_high2;
	double vpm_rate1;
	double vpm_rate2;
	
	int max_vpm1;
	int max_vpm2;
	
	double avg_wait;          //stores the average wait
	double density;           //stores the traffic density (actual VPM)
	double overall_avg_wait;
	double avg_wait_difference;
	

	//initialize local variables
	current_time = 0;
	avg_wait = 0;
	density = 0;
	
	//input the length of the simulation from std input
	cin  >> finish_time;
	
	//input the length of the simulation from std input
	cin  >> seed;	
	
	//input the length of the simulation from std input
	cin  >> controller_type;
	
	if (TIMER_TYPE == controller_type)
	{	//the controller type is timer
		//input the length of the timer interval for direction 1
		cin >> timer_interval;
	}
	else if (FUZZY_LOGIC_TYPE == controller_type)
	{
		//input the rule base from std input
		flc.loadFuzzySets(cin);
	
		//input the fuzzy sets from std input
		flc.loadRuleBase(cin);
	}

	//input the VPM type
	cin  >> vpm_type;
	
	if (VPM_RANDOM == vpm_type)
	{	//the VPM type is random
		cin >> max_vpm1;
		cin >> max_vpm2;
	}
	else if (VPM_FIXED == vpm_type)
	{	//the VPM type is fixed
		//input the first direction's fixed VPM
		cin  >> vpm1;

		//input the second direction's fixed VPM
		cin  >> vpm2;
	}
	else if (VPM_GRADUAL == vpm_type)
	{
		//the VPM type is gradual	
		//input the start VPM
		cin  >> vpm_low1;

		//input the stop VPM
		cin  >> vpm_high1;
		
		//input the VPM_change_rate
		cin  >> vpm_rate1;
		
		//the VPM type is gradual	
		//input the start VPM
		cin  >> vpm_low2;

		//input the stop VPM
		cin  >> vpm_high2;
		
		//input the VPM_change_rate
		cin  >> vpm_rate2;
	}
	else
	{
		while (!cin.eof())
		{
			int x;	
			cin >> x;
		}
	}
	

	//seed the random number generator
	srand(seed);
	
	//set the initial signal values
	d1.setSignal(GREEN);
	d2.setSignal(RED);
	
	//set the initial vpm values
	if (VPM_FIXED == vpm_type)
	{
		d1.setVPM(vpm1);
		d2.setVPM(vpm2);
	}
	else if (VPM_GRADUAL == vpm_type)
	{
		d1.setVPM(vpm_low1);
		d2.setVPM(vpm_low2);
	}

	
	
	
	//begin simulation
	while (current_time < finish_time)
	{
		if (VPM_RANDOM == vpm_type)
		{	//the vpm type is random
			//randomly generate two new vpm between 0 and max_vpm
			vpm1 = rand() % max_vpm1;
			vpm2 = rand() % max_vpm2;
			//set the generated vpm's to the directions
			d1.setVPM(vpm1);
			d2.setVPM(vpm2);
		}
		//else if (VPM_GRADUAL == vpm_type)
		//{	//the vpm is gradual
		//		
		//}	
		
		d1.doStuff(); //do calculations for direction 1
		d2.doStuff(); //do calculations for direction 2
	
		if (FUZZY_LOGIC_TYPE == controller_type)
		{	
			//the type being used is a fuzzy logic controller
			//determine inputs for the controller
			if (d1.signalRed())
			{
				avg_wait = d1.getAverageWait();
				density  = d2.getTrafficDensity();
			}
			else
			{
				avg_wait = d2.getAverageWait();
				density  = d1.getTrafficDensity();
			}
			//ask the controller what to do
			if (flc.compute(avg_wait, density) == CHANGE)
			{	//the controller said change
				//determine which signal is currently red
				if (d1.signalRed())
				{	//direction 1's signal is red
					//switch signals
					d1.setSignal(GREEN);
					d2.setSignal(RED);
				}//end if
				else
				{	//direction 2's signal is red
					//switch signals
					d1.setSignal(RED);
					d2.setSignal(GREEN);
				}//end else
			}//end if
		}//end if
		else if (TIMER_TYPE == controller_type)
		{	//the type is a timer
			//determine if a change is necessary
			if ((current_time % timer_interval) == 0)
			{	//it is time for a change
				//determine which signal is currently red
				if (d1.signalRed())
				{	//direction 1's signal is red
					//switch signals
					d1.setSignal(GREEN);
					d2.setSignal(RED);
				}//end if
				else
				{	//direction 2's signal is red
					//switch signals
					d1.setSignal(RED);
					d2.setSignal(GREEN);
				}//end else
			}//end if
		}//end else if
		
		//increment the time
		++current_time;
		
	} //loop while simiulation not complete
	
	//simulation complete
	
	//output results
	overall_avg_wait  = (double) (d1.getTotalAverageWait() * (double) d1.getTotalThroughput());
	overall_avg_wait += (double) (d2.getTotalAverageWait() * (double) d2.getTotalThroughput());
	overall_avg_wait /= (double) (d1.getTotalThroughput() + d2.getTotalThroughput());
	
	avg_wait_difference = fabs(d1.getTotalAverageWait() - d2.getTotalAverageWait());
	
//	cout << std::setiosflags( std::ios::fixed );
//	cout << std::setiosflags( std::ios::showpoint );
	cout << d1.getTotalAverageWait() << endl;
	cout << d2.getTotalAverageWait() << endl;
	cout << overall_avg_wait         << endl;
	cout << avg_wait_difference      << endl;
	
	return 0;
}
