//FuzzyLogicController.cpp

#include "FuzzyLogicController.h"

template <class T1, class T2, class T3>
FuzzyLogicController<T1,T2,T3>::FuzzyLogicController()
{
}


template <class T1, class T2, class T3>
FuzzyLogicController<T1,T2,T3>::~FuzzyLogicController()
{
	A_sets.clear();
	B_sets.clear();	
}


template <class T1, class T2, class T3>
T3 FuzzyLogicController<T1,T2,T3>::compute(double a, double b)
{
	vector<Rule<T1,T2,T3> > rules;
	vector<Rule<T1,T2,T3> > fire_vec;
	double                  current_fireStrength = 0;
	T3                      retval;
	int                     change_count = 0;
	
	//create a vector of all possible rule compinations with each membership value.
	//loop through the fuzzy sets for argument A
	for (vector<FuzzySet<FuzzyInput_A> >::iterator A_set = A_sets.begin(); A_set < A_sets.end(); ++A_set)
	{
		//get membership for A in this set
		double A_membership = (*A_set).getMembership(a);
		//if membership for Ais > 0
		if (A_membership > 0)
		{
			//loop through the fuzzy sets for argument B
			for (vector<FuzzySet<FuzzyInput_B> >::iterator B_set = B_sets.begin(); B_set < B_sets.end(); ++B_set)
			{
				//get membership value for B in this set
				double B_membership = (*B_set).getMembership(b);
				//if membership for B > 0
				if (B_membership > 0)
				{
					//construct a rule in the rule vector with the membership values
					T1 A_id = (*A_set).getId();
					T2 B_id = (*B_set).getId();
					Rule<T1,T2,T3> r(A_id, B_id, A_membership, B_membership);
					rules.push_back(r);
				}//end if
			}//end for
		}//end if
	}//end for
	
	
	//create a vector of the maximum rule fire strengths.
	//loop through the rule vector
	for (vector<Rule<FuzzyInput_A, FuzzyInput_B, FuzzyOutput> >::iterator r_iter = rules.begin(); r_iter < rules.end(); ++r_iter)
	{
		//if fire strength vector size is 0 
		if (fire_vec.empty())
		{
			//add this element to the fire strength vector
			fire_vec.push_back(*r_iter);
			//recalculate the current_fireStrength
			current_fireStrength = fire_vec.begin()->getFireStrength();
		}//end if
		//if this fire strength is > fire strength vector element's fire strength OR fire strength vector size is 0
		else if ( (*r_iter).getFireStrength() > current_fireStrength)
		{
			//clear fire strength vector
			fire_vec.clear();
			//add this element to the fire strength vector
			fire_vec.push_back(*r_iter);
			//recalculate the current_fireStrength
			current_fireStrength = fire_vec.begin()->getFireStrength();
		}//end else if
		//else if this fire strengh equals fire strength vector element's fire strengh
		else if ( (*r_iter).getFireStrength() == current_fireStrength)
		{
			//add this element to the fire strength vector
			fire_vec.push_back(*r_iter);
		}//end else if
	}//end for			


//use secondary rules to determine the actual output value.
//	loop through fire strength vector
	for (vector<Rule<FuzzyInput_A, FuzzyInput_B, FuzzyOutput> >::iterator r_iter = fire_vec.begin(); r_iter < fire_vec.end(); ++r_iter)
	{
		//test if rules output is to change
		if (rb.getOutput(*r_iter) == CHANGE)
			//increment the number of rules asking for change
			++change_count;
		else
			//cancel 1 rule asking for change by decrementing the number of rules asking for change
			--change_count;
	}
	
	if (change_count > 0)
		retval = CHANGE;
	else
		retval = NOCHANGE;
		
	//return the output value
	return retval;
}



template <class T1, class T2, class T3>
void FuzzyLogicController<T1,T2,T3>::loadFuzzySets(std::istream &in)
{
	T1 idA;
	T2 idB;
	int s;
	int e;
	int p;
	
	for (int i = 0; i < FuzzyInput_A_Count; ++i)
	{
		in >> idA;
		in >> s;
		in >> e;
		in >> p;
	
		A_sets.push_back(FuzzySet<T1>(idA, s, e, p));
	}
	
	for (int i = 0; i < FuzzyInput_B_Count; ++i)
	{
		in >> idB;
		in >> s;
		in >> e;
		in >> p;
	
		B_sets.push_back(FuzzySet<T2>(idB, s, e, p));
	}
	
}


template <class T1, class T2, class T3>
void FuzzyLogicController<T1,T2,T3>::loadRuleBase(std::istream &in)
{
	rb.setRuleBase(in);
}

#if #debug(yes)
template <class T1, class T2, class T3>
void FuzzyLogicController<T1,T2,T3>::showRuleBase()
{
	rb.show();
}

template <class T1, class T2, class T3>
void FuzzyLogicController<T1,T2,T3>::showFuzzySets()
{
	std::cout <<"Fuzzy Set A:" << std::endl;
	for (vector<FuzzySet<FuzzyInput_A> >::iterator A_set = A_sets.begin(); A_set < A_sets.end(); ++A_set)
	{
			(*A_set).show();
			std::cout << std::endl;
	}
	std::cout <<"Fuzzy Set B:" << std::endl;
	for (vector<FuzzySet<FuzzyInput_B> >::iterator B_set = B_sets.begin(); B_set < B_sets.end(); ++B_set)
	{
			(*B_set).show();
			std::cout << std::endl;
	}
}
#endif
